@@ -1,15 +1,16 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 #
 # Copyright (c) 2012-2021 Snowflake Computing Inc. All rights reserved.
 #
 
+from __future__ import annotations
+
 import logging
 import os
 import re
 import traceback
 from logging import getLogger
-from typing import TYPE_CHECKING, Dict, Optional, Type, Union
+from typing import TYPE_CHECKING
 
 from .compat import BASE_EXCEPTION_CLASS
 from .description import CLIENT_NAME, SNOWFLAKE_CONNECTOR_VERSION
@@ -34,13 +35,13 @@ class Error(BASE_EXCEPTION_CLASS):
 
     def __init__(
         self,
-        msg: Optional[str] = None,
-        errno: Optional[int] = None,
-        sqlstate: Optional[str] = None,
-        sfqid: Optional[str] = None,
-        done_format_msg: Optional[bool] = None,
-        connection: Optional["SnowflakeConnection"] = None,
-        cursor: Optional["SnowflakeCursor"] = None,
+        msg: str | None = None,
+        errno: int | None = None,
+        sqlstate: str | None = None,
+        sfqid: str | None = None,
+        done_format_msg: bool | None = None,
+        connection: SnowflakeConnection | None = None,
+        cursor: SnowflakeCursor | None = None,
     ):
         super().__init__(msg)
         self.msg = msg
@@ -109,7 +110,7 @@ class Error(BASE_EXCEPTION_CLASS):
 
         return "".join(traceback.format_list(filtered_frames))
 
-    def telemetry_msg(self) -> Optional[str]:
+    def telemetry_msg(self) -> str | None:
         if self.sqlstate != "n/a":
             return f"{self.errno:06d} ({self.sqlstate})"
         elif self.errno != -1:
@@ -117,42 +118,47 @@ class Error(BASE_EXCEPTION_CLASS):
         else:
             return None
 
-    def generate_telemetry_exception_data(self) -> Dict[str, str]:
+    def generate_telemetry_exception_data(self) -> dict[str, str]:
         """Generate the data to send through telemetry."""
+
         telemetry_data = {
-            TelemetryField.KEY_DRIVER_TYPE: CLIENT_NAME,
-            TelemetryField.KEY_DRIVER_VERSION: SNOWFLAKE_CONNECTOR_VERSION,
+            TelemetryField.KEY_DRIVER_TYPE.value: CLIENT_NAME,
+            TelemetryField.KEY_DRIVER_VERSION.value: SNOWFLAKE_CONNECTOR_VERSION,
         }
         telemetry_msg = self.telemetry_msg()
         if self.sfqid:
-            telemetry_data[TelemetryField.KEY_SFQID] = self.sfqid
+            telemetry_data[TelemetryField.KEY_SFQID.value] = self.sfqid
         if self.sqlstate:
-            telemetry_data[TelemetryField.KEY_SQLSTATE] = self.sqlstate
+            telemetry_data[TelemetryField.KEY_SQLSTATE.value] = self.sqlstate
         if telemetry_msg:
-            telemetry_data[TelemetryField.KEY_REASON] = telemetry_msg
+            telemetry_data[TelemetryField.KEY_REASON.value] = telemetry_msg
         if self.errno:
-            telemetry_data[TelemetryField.KEY_ERROR_NUMBER] = str(self.errno)
+            telemetry_data[TelemetryField.KEY_ERROR_NUMBER.value] = str(self.errno)
 
-        telemetry_data[TelemetryField.KEY_STACKTRACE] = SecretDetector.mask_secrets(
-            self.telemetry_traceback
-        )
+        telemetry_data[
+            TelemetryField.KEY_STACKTRACE.value
+        ] = SecretDetector.mask_secrets(self.telemetry_traceback)
 
         return telemetry_data
 
     def send_exception_telemetry(
         self,
-        connection: Optional["SnowflakeConnection"],
-        telemetry_data: Dict[str, str],
+        connection: SnowflakeConnection | None,
+        telemetry_data: dict[str, str],
     ) -> None:
         """Send telemetry data by in-band telemetry if it is enabled, otherwise send through out-of-band telemetry."""
+
         if (
             connection is not None
             and connection.telemetry_enabled
-            and not connection._telemetry.is_closed()
+            and not connection._telemetry.is_closed
         ):
             # Send with in-band telemetry
-            telemetry_data[TelemetryField.KEY_TYPE] = TelemetryField.SQL_EXCEPTION
-            telemetry_data[TelemetryField.KEY_EXCEPTION] = self.__class__.__name__
+            telemetry_data[
+                TelemetryField.KEY_TYPE.value
+            ] = TelemetryField.SQL_EXCEPTION.value
+            telemetry_data[TelemetryField.KEY_SOURCE.value] = connection.application
+            telemetry_data[TelemetryField.KEY_EXCEPTION.value] = self.__class__.__name__
             ts = get_time_millis()
             try:
                 connection._log_telemetry(TelemetryData(telemetry_data, ts))
@@ -160,14 +166,15 @@ class Error(BASE_EXCEPTION_CLASS):
                 logger.debug("Cursor failed to log to telemetry.", exc_info=True)
         elif connection is None:
             # Send with out-of-band telemetry
+
             telemetry_oob = TelemetryService.get_instance()
             telemetry_oob.log_general_exception(self.__class__.__name__, telemetry_data)
 
     def exception_telemetry(
         self,
         msg: str,
-        cursor: Optional["SnowflakeCursor"],
-        connection: Optional["SnowflakeConnection"],
+        cursor: SnowflakeCursor | None,
+        connection: SnowflakeConnection | None,
     ) -> None:
         """Main method to generate and send telemetry data for exceptions."""
         try:
@@ -178,16 +185,16 @@ class Error(BASE_EXCEPTION_CLASS):
                 self.send_exception_telemetry(connection, telemetry_data)
             else:
                 self.send_exception_telemetry(None, telemetry_data)
-        except Exception:  # NOQA
+        except Exception:
             # Do nothing but log if sending telemetry fails
             logger.debug("Sending exception telemetry failed")
 
     @staticmethod
     def default_errorhandler(
-        connection: "SnowflakeConnection",
-        cursor: "SnowflakeCursor",
-        error_class: Type["Error"],
-        error_value: Dict[str, str],
+        connection: SnowflakeConnection,
+        cursor: SnowflakeCursor,
+        error_class: type[Error],
+        error_value: dict[str, str],
     ) -> None:
         """Default error handler that raises an error.
 
@@ -212,9 +219,9 @@ class Error(BASE_EXCEPTION_CLASS):
 
     @staticmethod
     def errorhandler_wrapper_from_cause(
-        connection: "SnowflakeConnection",
-        cause: Union["Error", Exception],
-        cursor: Optional["SnowflakeCursor"] = None,
+        connection: SnowflakeConnection,
+        cause: Error | Exception,
+        cursor: SnowflakeCursor | None = None,
     ) -> None:
         """Wrapper for errorhandler_wrapper, it is called with a cause instead of a dictionary.
 
@@ -245,10 +252,10 @@ class Error(BASE_EXCEPTION_CLASS):
 
     @staticmethod
     def errorhandler_wrapper(
-        connection: Optional["SnowflakeConnection"],
-        cursor: Optional["SnowflakeCursor"],
-        error_class: Union[Type["Error"], Type[Exception]],
-        error_value: Dict[str, Union[str, bool, int]],
+        connection: SnowflakeConnection | None,
+        cursor: SnowflakeCursor | None,
+        error_class: type[Error] | type[Exception],
+        error_value: dict[str, str | bool | int],
     ) -> None:
         """Error handler wrapper that calls the errorhandler method.
 
@@ -280,9 +287,9 @@ class Error(BASE_EXCEPTION_CLASS):
 
     @staticmethod
     def errorhandler_wrapper_from_ready_exception(
-        connection: Optional["SnowflakeConnection"],
-        cursor: Optional["SnowflakeCursor"],
-        error_exc: Union["Error", Exception],
+        connection: SnowflakeConnection | None,
+        cursor: SnowflakeCursor | None,
+        error_exc: Error | Exception,
     ) -> None:
         """Like errorhandler_wrapper, but it takes a ready to go Exception."""
         if isinstance(error_exc, Error):
@@ -306,10 +313,10 @@ class Error(BASE_EXCEPTION_CLASS):
 
     @staticmethod
     def hand_to_other_handler(
-        connection: Optional["SnowflakeConnection"],
-        cursor: Optional["SnowflakeCursor"],
-        error_class: Union[Type["Error"], Type[Exception]],
-        error_value: Dict[str, Union[str, bool]],
+        connection: SnowflakeConnection | None,
+        cursor: SnowflakeCursor | None,
+        error_class: type[Error] | type[Exception],
+        error_value: dict[str, str | bool],
     ) -> bool:
         """If possible give error to a higher error handler in connection, or cursor.
 
@@ -330,9 +337,9 @@ class Error(BASE_EXCEPTION_CLASS):
 
     @staticmethod
     def errorhandler_make_exception(
-        error_class: Union[Type["Error"], Type[Exception]],
-        error_value: Dict[str, Union[str, bool]],
-    ) -> Union["Error", Exception]:
+        error_class: type[Error] | type[Exception],
+        error_value: dict[str, str | bool],
+    ) -> Error | Exception:
         """Helper function to errorhandler_wrapper that creates the exception."""
         error_value.setdefault("done_format_msg", False)
 
@@ -533,9 +540,7 @@ class MissingDependencyError(Error):
     """Exception for missing extras dependencies."""
 
     def __init__(self, dependency: str):
-        super(MissingDependencyError, self).__init__(
-            msg=f"Missing optional dependency: {dependency}"
-        )
+        super().__init__(msg=f"Missing optional dependency: {dependency}")
 
 
 class BindUploadError(Error):
